from . import MQTTGWHandler
from . import configs
from . import lorawanpacketdumper as pd
import json,time
class EndNode(MQTTGWHandler.MQTTGWHandler):
    """docstring for Endnode."""
    def __init__(self,config):
        super(EndNode, self).__init__(devGateway=config['devGateway'])
        #1.- Load the config
        self.DeviceEUI = config['devEUI']
        self.ApplicationEUI = config['appEUI']
        self.AppKey = config['appKey']
        #Flags
        self.IS_JOINED = False
        self.JOINOTAA_DEVNONCE = ""
        #Counters
        self.upframeCounter=0
        self.downframeCounter=0
        #Keys
        self.AppSKey = ""
        self.NwkSKey = ""
        self.DevAddr = ""
        #2.- We make some user configs
        self.setup()
        while not self.IS_JOINED:
            print('Trying to join the device ...')
            self.JoinOTAA()
            time.sleep(10)

        #3.- We call the main loop
        self.callLoop()

    def setup(self):
        pass
    def callLoop(self):
        while True:
            self.loop()

    def loop(self):
        pass

    def onRadioMessage(self,message):
        pass

    def on_message(self,client, userdata, msg):
        message = json.loads(str(msg.payload.decode("utf-8")))
        if self.IS_JOINED and message['type'] == 'Pull Response':
            payload = json.loads(message['data'])['txpk']['data']
            self.onRadioMessage(pd.src.packetDecoder.decryptDownLinkUtf(AppSKey=self.AppSKey,NwkSKey=self.NwkSKey,Payload=payload))
            return True
        if not self.IS_JOINED and message['type'] == 'Pull Response':
            payload = json.loads(message['data'])['txpk']['data']
            self._manageJoinAccept(payload)
            return True
        else:
            self.onRadioMessage(message)

    #***> OTAA JOIN ****************************
    def JoinOTAA(self):
        otaaMSG = pd.src.packetComposer.genOTAAJoinMessage(ApplicationEUI_msb = self.ApplicationEUI,
                                                           DeviceEUI_msb = self.DeviceEUI,
                                                           AppKey_msb = self.AppKey)

        self.JOINOTAA_DEVNONCE = otaaMSG['devNonce']
        self.rawsend(otaaMSG['message'])

    def _manageJoinAccept(self,payload):
        #We compute the sKeys
        hexmessage = pd.src.util.base642hexString(payload)
        skeys = pd.src.packetDecoder.genSessionKeys(AppKey=self.AppKey,JoinRequestDevNonce=self.JOINOTAA_DEVNONCE,Payload=hexmessage)
        self.AppSKey = skeys['appSKey']
        self.NwkSKey = skeys['nwkSKey']
        self.DevAddr = skeys['MACPayload']['DevAddr']
        self.IS_JOINED = True
        print("Joined")
        print(".................................")
        print("AppSKey:" + skeys['appSKey'])
        print("NwkSKey:" + skeys['nwkSKey'])

    #***> UPLINK MESSAGES ****************************
    def rawsend(self,message):
        self.produceOnMQTT(configs.uplinktopic, message)

    def sendUnconfirmedMessage(self,message):
        if not self.IS_JOINED:
            return False

        if self.upframeCounter > 65535:
            self.upframeCounter = 0
            self.JoinOTAA()

        hexmessage = pd.src.packetComposer.genUplinkMessage(schema={
            "MType" : "Unconfirmed Data Up",
            "DevAddr": self.DevAddr,
            "FCnt": self.upframeCounter,
            "FRMPayload": message,
            "FPort":1,
        },
        AppSKey=self.AppSKey,
        NwkSKey=self.NwkSKey)
        self.upframeCounter += 1

        self.rawsend(hexmessage)
        return True

    #***> DOWNLINK MESSAGES ****************************
