from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad
import binascii,math
from . import util

def encryptAES_ECB(hexStringKey,hexStringMessage,padding=0):
    key = binascii.unhexlify(hexStringKey)
    cipher = AES.new(key, AES.MODE_ECB)
    if padding == 0:
        cyf = cipher.encrypt(binascii.unhexlify(hexStringMessage))
    else:
        cyf = cipher.encrypt(pad(binascii.unhexlify(hexStringMessage),padding))
    return binascii.b2a_hex(cyf).decode("utf-8").upper()

def _createBlock(packet,i,dir):
    '''Creates Ai Block as described in LoRaWAN v1.0.3 Protocol section 4.3.3
    @param packet <models.FrameDef hexString>
    @param i the block number to code
    @dir 'up' for uplink, 'down' for downlink
    '''
    #We set the link dir
    Dir = '00'
    if dir == 'up':
        Dir = '00'
    elif dir == 'down':
        Dir = '01'
    else:
        #TODO throw exception
        print('Bad value')

    #We compose the block
    buffer = '0100000000' # As describe specifications
    buffer += Dir   # Direction Up or Down
    buffer += packet['MACPayload']['FHDR']['DevAddr']
    buffer += packet['MACPayload']['FHDR']['FCnt']
    buffer += '0000' #FCntUp or FCntDown, we append 2 lsb bytes because we have converted the counter
                     #from msb 2 lsb
    buffer += '00'   #0x00 as indicates the specifications
    buffer += util.integer21hexString(i + 1) #The block number
    return buffer

def encryptMACPayload(NwkSKey,AppSKey,encoded,updown):
    ''' Encrypts the uplink messages
    @param encoded is a json { "schema":<models.FrameDef hexString>, "FPort":port }
    '''
    #1.- We Select the crypto Key
    key=AppSKey
    if encoded['FPort'] == 0:
        key = NwkSKey
    frame = encoded['schema']
    #2.- We create Secuence S:
    SecuenceS = ''
    total_bolcks = math.ceil(len(frame['MACPayload']['FRMPayload'])/32)
    for i in range(total_bolcks):
        SecuenceS +=  _createBlock(frame,i,updown)
    encryptedSecuenceS = encryptAES_ECB(key,SecuenceS)
    #3.- Now we make the XOR functionc
    xor = util.hexStringXOR(encryptedSecuenceS,frame['MACPayload']['FRMPayload'])
    return xor
