from Crypto.Hash import CMAC
from Crypto.Cipher import AES
import base64,binascii,random
from . import util
import math

def calcJoinRequestMIC(hexStringMessage,appKey):
    '''Calcs the Join Request MIC
    @param hexstringmessage the hex string with  the message which we want to calc the mic
    @param appKey the LoraWAN AppKey given, for exaple, by TTN
    @return HexString with  MIC
    '''
    key = binascii.unhexlify(appKey)
    cobj = CMAC.new(key, ciphermod=AES)
    cobj.update(binascii.unhexlify(hexStringMessage))
    return cobj.hexdigest()[0:8] #4 bytes in HEX string format

def _createFrameBlock(packet,messageLen,dir):
    #We set the link dir
    Dir = '00'
    if dir == 'up':
        Dir = '00'
    elif dir == 'down':
        Dir = '01'
    else:
        #TODO throw exception
        print('Bad value')
    #We compose the block
    buffer = '4900000000' # As describe specifications
    buffer += Dir   # Direction Up or Down
    buffer += packet['MACPayload']['FHDR']['DevAddr']
    buffer += packet['MACPayload']['FHDR']['FCnt']
    buffer += '0000' #FCntUp or FCntDown, we append 2 lsb bytes because we have converted the counter
                     #from msb 2 lsb
    buffer += '00'   #0x00 as indicates the specifications
    buffer += util.integer21hexString(messageLen) #The block number
    return buffer

def calcFrameMIC(packet,hexStringMessage,NwkSKey,dir):
    #1.- First we calculate the MIC Block
    messageLen = math.ceil(len(hexStringMessage)/2)
    block = _createFrameBlock(packet,messageLen,dir)
    #2.- Now we calculate the MIC
    return calcJoinRequestMIC(block + hexStringMessage, NwkSKey).upper()
