from . import mic,util
from . import models as mod
from . import crypton
import copy

#--- JOIN REQUEST ----
def genOTAAJoinMessage(ApplicationEUI_msb = "", DeviceEUI_msb = "", AppKey_msb=""):
    ''' Returns the JOIN request message as Hex String'''
    ApplicationEUI_lsb = util.msb2lsb(ApplicationEUI_msb)
    DeviceEUI_lsb = util.msb2lsb(DeviceEUI_msb)
    devnonce = util.genRandomBytes(2)
    joinmsg = "00"
    joinmsg += ApplicationEUI_lsb #Join request field
    joinmsg += DeviceEUI_lsb      #Join request field
    joinmsg += devnonce           #Join request field
    joinmsg += mic.calcJoinRequestMIC(joinmsg,AppKey_msb)
    return {"message": joinmsg.upper(), "devNonce": util.msb2lsb(devnonce) }

#--- UPLINK REQUEST ----
def _loadFrameMessage(NewSchema):
    ''' Loads the user json message schema '''
    defaultSchema = mod.models.frameModels['FrameDef']
    baseSchema = copy.deepcopy( mod.models.frameModels['FrameDef'])
    baseSchema['MHDR']['MType'] =util.ifExists('MType',NewSchema,defaultSchema['MHDR']['MType'])
    baseSchema['MACPayload']['FHDR']['DevAddr'] = util.ifExists('DevAddr',NewSchema,defaultSchema['MACPayload']['FHDR']['DevAddr'])
    baseSchema['MACPayload']['FHDR']['FCnt'] = util.ifExists('FCnt',NewSchema,defaultSchema['MACPayload']['FHDR']['FCnt'])
    baseSchema['MACPayload']['FRMPayload'] = util.ifExists('FRMPayload',NewSchema,defaultSchema['MACPayload']['FRMPayload'])

    baseSchema['MACPayload']['FHDR']['FCtrl']['ADR'] = util.ifExists('FCtrl.ADR',NewSchema,defaultSchema['MACPayload']['FHDR']['FCtrl']['ADR'])
    baseSchema['MACPayload']['FHDR']['FCtrl']['ACK'] = util.ifExists('FCtrl.ACK',NewSchema,defaultSchema['MACPayload']['FHDR']['FCtrl']['ACK'])
    baseSchema['MACPayload']['FHDR']['FCtrl']['ADRACKReq'] = util.ifExists('FCtrl.ADRACKReq',NewSchema,defaultSchema['MACPayload']['FHDR']['FCtrl']['ADRACKReq'])
    baseSchema['MACPayload']['FHDR']['FCtrl']['FPending'] = util.ifExists('FCtrl.FPending',NewSchema,defaultSchema['MACPayload']['FHDR']['FCtrl']['FPending'])

    baseSchema['MACPayload']['FPort'] = util.ifExists('FPort',NewSchema,defaultSchema['MACPayload']['FPort'])
    return baseSchema

def _encodeFrameMessage(jsonFrame):
    ''' Encodes a JSON Structure As Hex Message '''
    baseSchema = copy.deepcopy( mod.models.frameModels['FrameDef'])

    port=jsonFrame['MACPayload']['FPort']

    #1.- We compose the MHDR ***
    mtype = jsonFrame['MHDR']['MType'].strip().replace(' ','').lower()
    baseSchema['MHDR']['MType'] = "40"
    if mtype == "unconfirmeddataup":
        baseSchema['MHDR']['MType'] = "40"
    elif mtype == "confirmeddataup":
        baseSchema['MHDR']['MType'] = "80"
    else:
        print("[ERROR in message encoding] Unknown MType: " + typ)
        return None
    #2.- We compose the MACPayload ***
    #2.1- FHDR
    baseSchema['MACPayload']['FHDR']['DevAddr'] = util.msb2lsb(jsonFrame['MACPayload']['FHDR']['DevAddr'])
    baseSchema['MACPayload']['FHDR']['FCtrl'] = "20"
    baseSchema['MACPayload']['FHDR']['FCnt'] = util.msb2lsb(util.integer22hexString(jsonFrame['MACPayload']['FHDR']['FCnt']))
    baseSchema['MACPayload']['FHDR']['FOpts'] = ""
    #2.2- FPort
    baseSchema['MACPayload']['FPort'] = util.integer21hexString(port) #1b
    #2.3- FRMPayload
    baseSchema['MACPayload']['FRMPayload'] = util.text2hexString(jsonFrame['MACPayload']['FRMPayload'])
    return { "schema":baseSchema, "FPort":port }

def genUplinkMessage(AppSKey="",NwkSKey="",schema={}):
    ''' Generates an uplink message '''
    #1.- We encode as HexString the user request
    encoded = _encodeFrameMessage(_loadFrameMessage(schema))
    #2.- We encrycpt the MACPayload
    crypted = crypton.encryptMACPayload(NwkSKey,AppSKey,encoded,'up')
    #3.- Now we calc the MIC and compose the message
    frame = encoded['schema']['MHDR']['MType']
    frame += util.serialize(encoded['schema']['MACPayload']['FHDR'])
    frame += encoded['schema']['MACPayload']['FPort']
    frame += crypted
    frame += mic.calcFrameMIC(encoded['schema'],frame,NwkSKey,'up')
    return frame
